@@ -1,4 +1,6 @@
 # -*- coding: utf-8 -*-
+# Copyright 2020- Datastax, Inc. All rights reserved.
+# Copyright 2020 Spotify Inc. All rights reserved.
 # Copyright 2019 Spotify AB. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,6 +16,7 @@
 # limitations under the License.
 
 
+import fileinput
 import itertools
 import logging
 import os
@@ -31,6 +34,7 @@ from cassandra.cluster import Cluster, ExecutionProfile
 from cassandra.policies import WhiteListRoundRobinPolicy
 from cassandra.auth import PlainTextAuthProvider
 from ssl import SSLContext, PROTOCOL_TLSv1, CERT_REQUIRED
+from medusa.network.hostname_resolver import HostnameResolver
 
 
 class SnapshotPath(object):
@@ -51,6 +55,7 @@ class CqlSessionProvider(object):
         self._ip_addresses = ip_addresses
         self._auth_provider = None
         self._ssl_context = None
+        self._cassandra_config = cassandra_config
 
         if cassandra_config.cql_username is not None and cassandra_config.cql_password is not None:
             auth_provider = PlainTextAuthProvider(username=cassandra_config.cql_username,
@@ -92,7 +97,7 @@ class CqlSessionProvider(object):
             while attempts < max_retries:
                 try:
                     session = cluster.connect()
-                    return CqlSession(session)
+                    return CqlSession(session, self._cassandra_config.resolve_ip_addresses)
                 except Exception as e:
                     logging.debug('Failed to create session', exc_info=e)
                 time.sleep(delay)
@@ -100,7 +105,7 @@ class CqlSessionProvider(object):
             raise Exception('Could not establish CQL session after {attempts}'.format(attempts=attempts))
         else:
             session = cluster.connect()
-            return CqlSession(session)
+            return CqlSession(session, self._cassandra_config.resolve_ip_addresses)
 
 
 class Nodetool(object):
@@ -128,8 +133,9 @@ class Nodetool(object):
 class CqlSession(object):
     EXCLUDED_KEYSPACES = ['system_traces']
 
-    def __init__(self, session):
+    def __init__(self, session, resolve_ip_addresses=True):
         self._session = session
+        self.hostname_resolver = HostnameResolver(resolve_ip_addresses)
 
     def __enter__(self):
         return self
@@ -159,13 +165,13 @@ class CqlSession(object):
 
     def datacenter(self):
         logging.debug('Checking datacenter...')
-        listen_address = self.cluster.contact_points[0]
+        listen_address = socket.gethostbyname(self.cluster.contact_points[0])
         token_map = self.cluster.metadata.token_map
 
         for host in token_map.token_to_host_owner.values():
-            socket_host = socket.gethostbyname(listen_address)
+            socket_host = self.hostname_resolver.resolve_fqdn(listen_address)
             logging.debug('Checking host {} against {}/{}'.format(host.address, listen_address, socket_host))
-            if host.address == listen_address or host.address == socket_host:
+            if host.address == listen_address or self.hostname_resolver.resolve_fqdn(host.address) == socket_host:
                 return host.datacenter
 
         raise RuntimeError('Unable to current datacenter')
@@ -191,7 +197,7 @@ class CqlSession(object):
         host_tokens_pairs = [(host, list(map(get_token, tokens))) for host, tokens in host_tokens_groups]
 
         return {
-            socket.gethostbyaddr(host.address)[0]: {
+            self.hostname_resolver.resolve_fqdn(host.address): {
                 'tokens': tokens,
                 'is_up': host.is_up
             }
@@ -254,17 +260,36 @@ class CassandraConfigReader(object):
             if self._config['listen_address']:
                 return self._config['listen_address']
 
-        return socket.gethostname()
+        return socket.gethostbyname(socket.getfqdn())
 
     @property
     def storage_port(self):
         if 'storage_port' in self._config:
             if self._config['storage_port']:
                 return self._config['storage_port']
-            else:
-                return "7000"
-        else:
-            return "7000"
+        return "7000"
+
+    @property
+    def native_port(self):
+        if 'native_transport_port' in self._config:
+            if self._config['native_transport_port']:
+                return self._config['native_transport_port']
+        return "9042"
+
+    @property
+    def rpc_port(self):
+        if 'rpc_port' in self._config:
+            if self._config['rpc_port']:
+                return self._config['rpc_port']
+        return "9160"
+
+    @property
+    def seeds(self):
+        seeds = list()
+        if 'seed_provider' in self._config:
+            if self._config['seed_provider']:
+                return self._config.get('seed_provider')[0]['parameters'][0]['seeds'].replace(' ', '').split(',')
+        return seeds
 
 
 class Cassandra(object):
@@ -280,6 +305,7 @@ class Cassandra(object):
         logging.warning('is ccm : {}'.format(self._is_ccm))
 
         config_reader = CassandraConfigReader(cassandra_config.config_file)
+        self._cassandra_config_file = cassandra_config.config_file
         self._root = config_reader.root
         self._commitlog_path = config_reader.commitlog_directory
         self._saved_caches_path = config_reader.saved_caches_directory
@@ -289,6 +315,9 @@ class Cassandra(object):
             cassandra_config)
 
         self._storage_port = config_reader.storage_port
+        self._native_port = config_reader.native_port
+        self._rpc_port = config_reader.rpc_port
+        self.seeds = config_reader.seeds
 
     def _has_systemd(self):
         try:
@@ -316,10 +345,22 @@ class Cassandra(object):
     def saved_caches_path(self):
         return self._saved_caches_path
 
+    @property
+    def hostname(self):
+        return self._hostname
+
     @property
     def storage_port(self):
         return self._storage_port
 
+    @property
+    def native_port(self):
+        return self._native_port
+
+    @property
+    def rpc_port(self):
+        return self._rpc_port
+
     class Snapshot(object):
         def __init__(self, parent, tag):
             self._parent = parent
@@ -466,20 +507,8 @@ class Cassandra(object):
 
     def start(self, token_list):
         if self._is_ccm == 0:
-            jvm_opts = '-Dcassandra.initial_token={} -Dcassandra.auto_bootstrap=false'.format(','.join(token_list))
-            if self._os_has_systemd:
-                tokens_env = 'sudo systemctl set-environment JVM_OPTS="{}"'.format(jvm_opts)
-                cmd = '{} && {}'.format(tokens_env, ' '.join(shlex.quote(x) for x in self._start_cmd))
-            else:
-                tokens_env = 'sudo env JVM_OPTS="{}"'.format(jvm_opts)
-                # Have to use command line as Subprocess does not handle quotes well
-                # undoing 'shlex' split, back to a string in this case for '_start_cmd'
-                # joining the 2 pieces of the command
-                # Also, if the command to run cassandra uses sudo, we need to remove it
-                # to add it as the first element
-                if 'sudo' in self._start_cmd:
-                    self._start_cmd.remove('sudo')
-                cmd = '{} {}'.format(tokens_env, ' '.join(shlex.quote(x) for x in self._start_cmd))
+            self.replaceTokensInCassandraYamlAndDisableBootstrap(token_list)
+            cmd = '{}'.format(' '.join(shlex.quote(x) for x in self._start_cmd))
             logging.debug('Starting Cassandra with {}'.format(cmd))
             # run the command using 'shell=True' option
             # to interpret the string command well
@@ -487,30 +516,69 @@ class Cassandra(object):
         else:
             subprocess.check_output(self._start_cmd, shell=True)
 
+    def replaceTokensInCassandraYamlAndDisableBootstrap(self, token_list):
+        initial_token_line_found = False
+        auto_bootstrap_line_found = False
+        for line in fileinput.input(self._cassandra_config_file, inplace=True):
+            if (line.startswith("initial_token:")):
+                initial_token_line_found = True
+                print('initial_token: {}'.format(','.join(token_list)), end='\n')
+            elif (line.startswith("num_tokens:")):
+                print('num_tokens: {}'.format(len(token_list)), end='\n')
+            elif (line.startswith("auto_bootstrap:")):
+                auto_bootstrap_line_found = True
+                print('auto_bootstrap: false', end='\n')
+            else:
+                print('{}'.format(line), end='')
+
+        if (not initial_token_line_found):
+            with open(self._cassandra_config_file, "a") as cassandra_yaml:
+                cassandra_yaml.write('\ninitial_token: {}'.format(','.join(token_list)))
+
+        if (not auto_bootstrap_line_found):
+            with open(self._cassandra_config_file, "a") as cassandra_yaml:
+                cassandra_yaml.write('\nauto_bootstrap: false')
+
 
 def wait_for_node_to_come_up(config, host, retries=10, delay=6):
+    logging.info('Waiting for Cassandra to come up on %s', host)
+    check_function = is_node_up
+    if not _wait_for_node(config, host, check_function, retries, delay):
+        raise CassandraNodeNotUpError(host, retries)
+    logging.info('Cassandra is up on %s', host)
+
+
+def wait_for_node_to_go_down(config, host, retries=10, delay=6):
+    logging.info('Waiting for Cassandra to go down on %s', host)
+
+    def check_function(config, host):
+        return not is_node_up(config, host)
+
+    if not _wait_for_node(config, host, check_function, retries, delay):
+        # TODO can do a kill here
+        raise CassandraNodeNotDownError(host, retries)
+    logging.info('Cassandra is down %s', host)
+
+
+def _wait_for_node(config, host, check_function, retries, delay):
     """
-        Polls the node until the health check passes.
+        Polls the node until the check_function check passes.
 
-        :param health_check: The type of health check to perform, one of cql, thrift, all.
         :param host: The target host on which to run the check
+        :param check_function: called to determine node state
         :param retries: The number of times to retry the health check. Defaults to 10
         :param delay: A delay in seconds to wait before polling again. Defaults to 6 seconds.
-        :return: None when the node is determined to be up. If the retries are exhausted, an exception is raised.
-        """
-
-    logging.info('Waiting for Cassandra to come up on %s', host)
-
+        :return: True when the node is determined to be up. If the retries are exhausted, returns False
+    """
     attempts = 0
     while attempts < retries:
-        if is_node_up(config, host):
-            logging.info('Cassandra is up on %s', host)
-            return None
+        if check_function(config, host):
+            return True
         else:
             time.sleep(delay)
             attempts = attempts + 1
 
-    raise CassandraNodeNotUpError(host, attempts)
+    return False
 
 
 def is_node_up(config, host):
@@ -526,25 +594,33 @@ def is_node_up(config, host):
     must be ready to accept requests for both in order for the health check to be successful.
     """
 
+    health_check = config.checks.health_check
     if int(config.cassandra.is_ccm) == 1:
         args = ['ccm', 'node1', 'nodetool']
+        if health_check == 'thrift':
+            return is_ccm_up(args, 'statusthrift')
+        elif health_check == 'all':
+            return is_ccm_up(list(args), 'statusbinary') and is_ccm_up(list(args), 'statusthrift')
+        else:
+            return is_ccm_up(args, 'statusbinary')
     else:
-        args = Nodetool(config.cassandra).nodetool + ['-h', host]
-
-    health_check = config.restore.health_check
-    if health_check == 'cql':
-        return is_cql_up(args)
-    elif health_check == 'thrift':
-        return is_thrift_up(args)
-    elif health_check == 'all':
-        return is_cql_up(list(args)) and is_thrift_up(list(args))
-    else:
-        return is_cql_up(args)
+        cassandra = Cassandra(config.cassandra)
+        native_port = cassandra.native_port
+        rpc_port = cassandra.rpc_port
+        nc_timeout = 10
+        args = ['timeout', str(nc_timeout), 'nc', '-zv', host]
+        if health_check == 'thrift':
+            return is_cassandra_up(args, rpc_port)
+        elif health_check == 'all':
+            return is_cassandra_up(list(args), rpc_port) and is_cassandra_up(list(args), native_port)
+        else:
+            # cql only
+            return is_cassandra_up(args, native_port)
 
 
-def is_cql_up(args):
+def is_ccm_up(args, nodetool_command):
     try:
-        args.append('statusbinary')
+        args.append(nodetool_command)
         output = subprocess.check_output(args, stderr=subprocess.STDOUT, universal_newlines=True)
         return output.find('running') >= 0
     except subprocess.CalledProcessError as e:
@@ -553,19 +629,20 @@ def is_cql_up(args):
         return False
 
 
-def is_thrift_up(args):
+def is_cassandra_up(args, port):
     try:
-        args.append('statusthrift')
-        output = subprocess.check_output(args, stderr=subprocess.STDOUT, universal_newlines=True)
-        return output.find('running') >= 0
+        args.append(port)
+        subprocess.check_call(args)
+        # check_call returns if call succeeded (returncode is 0). If not exception is triggered
+        return True
     except subprocess.CalledProcessError:
-        logging.debug('The thrift server is not up yet')
+        logging.debug('The node is not up yet')
         return False
 
 
 class CassandraNodeNotUpError(Exception):
     """
-    Raised when it cannot be veriffied that a node is up by checking either nodetool statusbinary and/or
+    Raised when it cannot be verified that a node is up by checking either nodetool statusbinary and/or
     nodetool statusthrift
 
     Attributes:
@@ -576,3 +653,13 @@ class CassandraNodeNotUpError(Exception):
     def __init(self, host, attempts):
         msg = 'Could not verify that Cassandra is up on {host} after {attempts}'.format(host=host, attempts=attempts)
         super(CassandraNodeNotUpError, self).__init__(msg)
+
+
+class CassandraNodeNotDownError(Exception):
+    """
+    Raised when we give up waiting on a node to go down, meaning nodetool statusbinary and/or statusthrift keep
+    reporting open ports.
+    """
+    def __init(self, host, attempts):
+        msg = 'Could not verify that Cassandra is down on {host} after {attempts}'.format(host=host, attempts=attempts)
+        super(CassandraNodeNotDownError, self).__init__(msg)
