@@ -18,17 +18,17 @@ import json
 import logging
 import os
 import shlex
-import socket
 import subprocess
 import sys
 import time
 import uuid
 
-from medusa.cassandra_utils import Cassandra, is_node_up
+from medusa.cassandra_utils import Cassandra, is_node_up, wait_for_node_to_go_down
 from medusa.download import download_data
 from medusa.storage import Storage
 from medusa.verify_restore import verify_restore
-
+from medusa.network.hostname_resolver import HostnameResolver
+import medusa.config
 
 A_MINUTE = 60
 MAX_ATTEMPTS = 60
@@ -51,7 +51,8 @@ def restore_node(config, temp_dir, backup_name, in_place, keep_auth, seeds, veri
                                    keyspaces, tables)
 
     if verify:
-        verify_restore([socket.getfqdn()], config)
+        hostname_resolver = HostnameResolver(medusa.config.evaluate_boolean(config.cassandra.resolve_ip_addresses))
+        verify_restore([hostname_resolver.resolve_fqdn()], config)
 
 
 def restore_node_locally(config, temp_dir, backup_name, in_place, keep_auth, seeds, storage, keyspaces, tables):
@@ -82,11 +83,12 @@ def restore_node_locally(config, temp_dir, backup_name, in_place, keep_auth, see
 
     logging.info('Stopping Cassandra')
     cassandra.shutdown()
+    wait_for_node_to_go_down(config, cassandra.hostname)
 
     # Clean the commitlogs, the saved cache to prevent any kind of conflict
     # especially around system tables.
-    clean_path(cassandra.commit_logs_path)
-    clean_path(cassandra.saved_caches_path)
+    clean_path(cassandra.commit_logs_path, keep_folder=True)
+    clean_path(cassandra.saved_caches_path, keep_folder=True)
 
     # move backup data to Cassandra data directory according to system table
     logging.info('Moving backup data to Cassandra data directory')
@@ -118,6 +120,8 @@ def restore_node_locally(config, temp_dir, backup_name, in_place, keep_auth, see
     else:
         cassandra.start(tokens)
 
+    # Clean the restored data from local temporary folder
+    clean_path(download_dir, keep_folder=False)
     return node_backup
 
 
@@ -152,10 +156,13 @@ def restore_node_sstableloader(config, temp_dir, backup_name, in_place, keep_aut
         invoke_sstableloader(config, download_dir, keep_auth, fqtns_to_restore, cassandra.storage_port)
         logging.info('Finished loading backup from {}'.format(fqdn))
 
+    # Clean the restored data from local temporary folder
+    clean_path(download_dir, keep_folder=False)
     return node_backup
 
 
 def invoke_sstableloader(config, download_dir, keep_auth, fqtns_to_restore, storage_port):
+    hostname_resolver = HostnameResolver(medusa.config.evaluate_boolean(config.cassandra.resolve_ip_addresses))
     cassandra_is_ccm = int(shlex.split(config.cassandra.is_ccm)[0])
     keyspaces = os.listdir(str(download_dir))
     for keyspace in keyspaces:
@@ -169,7 +176,7 @@ def invoke_sstableloader(config, download_dir, keep_auth, fqtns_to_restore, stor
                     cql_username = 'foo' if config.cassandra.cql_username is None else config.cassandra.cql_username
                     cql_password = 'foo' if config.cassandra.cql_password is None else config.cassandra.cql_password
                     sstableloader_args = [config.cassandra.sstableloader_bin,
-                                          '-d', socket.getfqdn() if cassandra_is_ccm == 0
+                                          '-d', hostname_resolver.resolve_fqdn() if cassandra_is_ccm == 0
                                           else '127.0.0.1',
                                           '--username', cql_username,
                                           '--password', cql_password,
@@ -194,11 +201,10 @@ def invoke_sstableloader(config, download_dir, keep_auth, fqtns_to_restore, stor
                     output = subprocess.check_output(sstableloader_args)
                     for line in output.decode('utf-8').split('\n'):
                         logging.debug(line)
-    clean_path(download_dir)
 
 
 def keyspace_is_allowed_to_restore(keyspace, keep_auth, fqtns_to_restore):
-    if keyspace == 'system':
+    if keyspace == 'system' or keyspace == 'system_schema':
         return False
 
     if keyspace == 'system_auth' and keep_auth is True:
@@ -224,10 +230,19 @@ def table_is_allowed_to_restore(keyspace, table, fqtns_to_restore):
     return True
 
 
-def clean_path(p):
-    if p.exists():
-        logging.debug('Cleaning ({})'.format(p))
-        subprocess.check_output(['sudo', '-u', p.owner(), 'rm', '-rf', str(p)])
+def clean_path(p, keep_folder=False):
+    path = str(p)
+    if p.exists() and os.path.isdir(path) and len(os.listdir(path)):
+        logging.debug('Cleaning ({})'.format(path))
+        if keep_folder:
+            logging.debug('Removing files - keep folder {}'.format(path))
+            for f in os.listdir(path):
+                file_path = os.path.join(path, f)
+                logging.debug('Removing file {}'.format(file_path))
+                subprocess.check_output(['sudo', '-u', p.owner(), 'rm', '-rf', file_path])
+        else:
+            logging.debug('Remove folder {} and content'.format(path))
+            subprocess.check_output(['sudo', '-u', p.owner(), 'rm', '-rf', path])
 
 
 def maybe_restore_section(section, download_dir, cassandra_data_dir, in_place, keep_auth):
@@ -302,6 +317,7 @@ def wait_for_seeds(config, seeds):
 
 def get_fqtns_to_restore(keep_keyspaces, keep_tables, manifest_str):
 
+    SYSTEM_KEYSPACES = ['system', 'system_schema', 'system_auth', 'system_distributed']
     fqtns = set()
     manifest = json.loads(manifest_str)
 
@@ -319,8 +335,8 @@ def get_fqtns_to_restore(keep_keyspaces, keep_tables, manifest_str):
             fqtns.add(fqtn_with_id)
             continue
 
-        # if the whole keyspace is a keep
-        if ks in keep_keyspaces:
+        # if the whole keyspace is a keep, or a system keyspace (C* internal)
+        if ks in keep_keyspaces or ks in SYSTEM_KEYSPACES:
             fqtns.add(fqtn_with_id)
             continue
 
